@@ -1,5 +1,4 @@
-"""The :mod:`sklearn.inspection` module includes tools for model inspection."""
-
+"""Tools for model inspection."""
 
 from ._partial_dependence import partial_dependence
 from ._permutation_importance import permutation_importance
